searchNodes=[{"doc":"A utility for producing entities using business logic defined by your application. The main idea of SeedFactory is to produce entities in tests according to your application business logic (read as context functions if you use https://hexdocs.pm/phoenix/contexts.html ) whenever it is possible and avoid direct inserts to the database (opposed to ex_machina ). This approach allows to minimize testing of invalid states as you're not forced to keep complex database structure in your head in order to prepare test data. Context , Entities and commands are the core concepts of the library. Context is a map which can be populated by entities using commands. The schema with instructions on how commands modify context is described using DSL with the help of SeedFactory.Schema module. Commands can be used to: produce entity (put new data in the context) update entity (replace the existing entity in the context) delete entity (remove the entity from the context) An entity can be produced only by one command. When a command is executed, produced entities are assigned to the context using the name of the entity as a key. A command has params with instructions on how to generate arguments for a resolver if they are not passed explicitly with exec/3 function. The instruction can be either a zero-arity function or an atom. A zero-arity function is used for generating data. An atom is used to specify an entity which should be taken from the context. If a required entity cannot be found in a context, then SeedFactory automatically executes a corresponding command which produces the entity. Let's take a look at an example of a simple schema. defmodule MyApp.SeedFactorySchema do use SeedFactory.Schema command :create_company do param :name , &amp; Faker.Company . name / 0 resolve ( fn args -&gt; with { :ok , company } &lt;- MyApp.Companies . create_company ( args ) do { :ok , %{ company : company } } end end ) produce :company , from : :company end command :create_user do param :name , &amp; Faker.Person . name / 0 param :company , :company resolve ( fn args -&gt; MyApp.Users . create_user ( args . company , args . name ) end ) produce :user , from : :user produce :profile , from : :profile end end The schema above describes how to produce 3 entities ( :company , :user and :profile ) using 2 commands ( :create_user and :create_company ). In order to use the schema, we need to put metadata about it to the context using init/2 function: context = %{ } context = init ( context , MyApp.SeedFactorySchema ) If you use SeedFactory in tests, use SeedFactory.Test helper module instead. Now, we can use produce/2 to produce entities using commands defined in the schema: context = produce ( context , :user ) After executing the code above, context will have 3 new keys: :company , :user and :profile . SeedFactory automatically executes a chain of commands needed to produce :user entity. In this case :company was produced by the :create_company command before resolving the :create_user command. exec/3 can be used if you want to specify parameters explicitly: context = context |&gt; exec ( :create_company , name : &quot;GitHub&quot; ) |&gt; exec ( :create_user , name : &quot;John Doe&quot; ) exec/3 fails if produced entities are already present in the context. It is possible to rebind entities in order to assign them to the context with the different name: context = context |&gt; produce ( user : :user1 , profile : :profile1 ) |&gt; produce ( user : :user2 , profile : :profile2 ) The snippet above puts the following keys to the context: :company , :user1 , :profile1 , :user2 , :profile2 . The :company is shared in this case, so two users have different profiles and belong to the same company. Let's create 2 companies with 1 user in each and pass names to companies explicitly: context = context |&gt; rebind ( [ company : :company1 , user : :user1 , profile : :profile1 ] , fn context -&gt; context |&gt; exec ( :create_company , name : &quot;GitHub&quot; ) |&gt; produce ( :user ) end ) |&gt; rebind ( [ company : :company2 , user : :user2 , profile : :profile2 ] , fn context -&gt; context |&gt; exec ( :create_company , name : &quot;Microsoft&quot; ) |&gt; produce ( :user ) end )","ref":"SeedFactory.html","title":"SeedFactory","type":"module"},{"doc":"Executes a command and puts its result to context . Example iex&gt; context = %{ } ...&gt; context = init ( context , MySeedFactorySchema ) ...&gt; context = exec ( context , :create_user , first_name : &quot;John&quot; , last_name : &quot;Doe&quot; ) ...&gt; Map . take ( context . user , [ :first_name , :last_name ] ) %{ first_name : &quot;John&quot; , last_name : &quot;Doe&quot; }","ref":"SeedFactory.html#exec/3","title":"SeedFactory.exec/3","type":"function"},{"doc":"Puts metadata about schema to context , so context becomes usable by rebind/3 , produce/2 and exec/3 . Example iex&gt; context = %{ } ...&gt; init ( context , MySeedFactorySchema ) %{ ... }","ref":"SeedFactory.html#init/2","title":"SeedFactory.init/2","type":"function"},{"doc":"Produces entities by executing corresponding commands. Examples %{ user : _ } = produce ( context , :user ) %{ user : _ , company : _ } = produce ( context , [ :user , :company ] ) %{ user1 : _ , } = produce ( context , user : :user1 )","ref":"SeedFactory.html#produce/2","title":"SeedFactory.produce/2","type":"function"},{"doc":"Changes default behaviour of entities assignment. Modifies a context so entities engage with context using provided names. It helps to produce the same entity multiple times and assign to the context with different names. Example %{ company1 : _ , company2 : _ } = context |&gt; rebind ( [ company : :company1 ] , fn context -&gt; exec ( context , :create_company , name : &quot;GitHub&quot; ) end ) |&gt; rebind ( [ company : :company2 ] , fn context -&gt; exec ( context , :create_company , name : &quot;Microsoft&quot; ) end )","ref":"SeedFactory.html#rebind/3","title":"SeedFactory.rebind/3","type":"function"},{"doc":"","ref":"SeedFactory.html#t:context/0","title":"SeedFactory.context/0","type":"type"},{"doc":"","ref":"SeedFactory.html#t:entity_name/0","title":"SeedFactory.entity_name/0","type":"type"},{"doc":"","ref":"SeedFactory.html#t:rebinding_rule/0","title":"SeedFactory.rebinding_rule/0","type":"type"},{"doc":"A schema describes how commands modify context This module provides a DSL for defining schemas that describe how entities should be created, updated, or deleted within a context using the SeedFactory library. In order to use the DSL, add the following line to your module: use SeedFactory.Schema Command Definition To define a command, use the command macro followed by the command name. Inside the command block, you can define various parameters, a resolution, and produce, update, and delete directives. command :command_name do # Parameters, resolution, produce, update, and delete directives end Parameters Parameters define the inputs required for the command's resolver function. They can be defined using the param macro. param :param_name , atom_or_function The atom_or_function can be either a static atom or a zero-arity function that generates dynamic data. When using an atom as the value, it refers to an entity within the context. Resolution The resolution defines the logic to be executed when the command is invoked. It is implemented as a resolver function inside the resolve block. The resolver function is an anonymous function that takes args as its parameter, representing the arguments provided when invoking the command. resolve ( fn args -&gt; # Resolver logic end ) The resolver function should return {:ok, map} , where map keys are atoms and values represent entities. The atom keys will be used by the :from option in produce and update directives. Producing Entities The produce directive specifies that the command will put a new entity to the context. It takes two arguments: the name of the entity being produced and options. Options :from - an atom that specifies the key of the map returned by the resolver. produce :entity_name , from : :source_key Updating Entities The update directive modifies an existing entity within the context. It takes two arguments: the name of the entity being updated and options. Options :from - an atom that specifies the key of the map returned by the resolver. update :entity_name , from : :source_key Deleting Entities The delete directive removes an entity from the context. delete :entity_name It is used to delete an entity from the context.","ref":"SeedFactory.Schema.html","title":"SeedFactory.Schema","type":"module"},{"doc":"A helper module for ExUnit . Usage Add the following line to your test modules: use SeedFactory.Test , schema : MySeedFactorySchema It sets up SeedFactory by invoking SeedFactory.init/2 and imports SeedFactory.rebind/3 , SeedFactory.produce/2 , SeedFactory.exec/3 and SeedFactory.exec/2 functions.","ref":"SeedFactory.Test.html","title":"SeedFactory.Test","type":"module"},{"doc":"A macro that implicitly passes context and allows using SeedFactory.produce/2 outside the test block. Basically, it creates a setup block and calls SeedFactory.produce/2 inside. Examples produce :company test &quot;my test&quot; , company : company do assert my_function ( company ) end produce [ :user , :project ] test &quot;my test&quot; , user : user , project : project do assert my_function ( project , user ) end produce org : :org1 produce org : :org2 test &quot;my test&quot; , org1 : org1 , org2 : org2 do assert my_function ( org2 , org1 ) end","ref":"SeedFactory.Test.html#produce/1","title":"SeedFactory.Test.produce/1","type":"macro"}]